/* 
 * File:   differential_equations.h
 * Author: aj373
 *
 * Created on April 25, 2018, 4:23 PM
 */

#include "plib.h"

// config.h sets 40 MHz
#define	SYS_FREQ 40000000

#define DAC_config_chan_A 0b0011000000000000
#define DAC_config_chan_B 0b1011000000000000
#define Fs 200000.0
#define two32 4294967296.0 // 2^32 

#define five_ms 80
#define sixtyfive_ms 1040

// === 16:16 fixed point macros ========================================== CAN PUT IN CARDIAC NEURON.h
typedef signed int fix16 ;
#define multfix16(a,b) ((fix16)(((( signed long long)(a))*(( signed long long)(b)))>>16)) //multiply two fixed 16:16
#define float2fix16(a) ((fix16)((a)*65536.0)) // 2^16
#define fix2float16(a) ((float)(a)/65536.0)
#define fix2int16(a)    ((int)((a)>>16))
#define int2fix16(a)    ((fix16)((a)<<16))
#define divfix16(a,b) ((fix16)((((signed long long)(a)<<16)/(b)))) 
#define sqrtfix16(a) (float2fix16(sqrt(fix2float16(a)))) 
#define absfix16(a) abs(a)
#define onefix16 0x00010000 // int2fix16(1)

//sine lookup tables
#define sine_table_size 256


extern volatile double timeCounter; // NECESSARY??
extern double deltaTime; // NECESSARY?

// Concentration and conductance externs for TFT Output
extern double GNA, GCA, GK;
extern double NAe, CAe, Ke;



double alpha_d(double voltage);
double alpha_f(double voltage);
double alpha_k1(double v);
double alpha_m(double voltage);
double alpha_r(double v);
double alpha_to(double v);

double beta_d(double voltage);
double beta_f(double v);
double beta_k1(double v);
double beta_m(double voltage);
double beta_r(double v);
double beta_to(double v);

double calcNaCaCurrent( double v, double T, double gNaCa, double KmNa, double Nae, double Nai, double KmCa, double ksat, double Cae, double Cai, double eta );
double calcNaKCurrent( double v, double sigma , double T, double gNaK );

double E_to(double Nae, double Ke, double Nai, double Ki,double T);
double equilibriumPotential(double intra,double extra,double N,double T);
double expDTChange(double DT, double tau);
double f_Ca(double Cai , double KCa);
double fNaK(double v, double sigma , double T);
double f_primeNaK(double Ke, double Nai);
double gateSteadyState(double alpha,double beta);
double gateTau( double alpha,double beta );
double getRateOfChangeK( double v, double DT, double tau_x );
double sigma_nak(double Nae);

double tau_v( double v );
double tau_x( double v );
double v_inf(double v);
double X_inf(double v);

// Called from outside
void calculateEquationConstants();
void equationInit();
void updatePreviousParameters();
void updateNextParameters();
void updateTimeCurrentVoltage();