#include "plib.h"
#include "math.h"
#include "stdio.h"
#include "tft_master.h"
#include "helper_functions.h"
#include "Cardiac_neuron.h"
#include "differential_equations.h"

// adc thresholding variables
int adc_10;
float V;
fix16 Vfix;
// adc threading variables
fix16 ADC_scale = float2fix16(3.3/1023.0); //Vref/(full scale)

// screen variables
int clearScreen = 0;
int setRed = 10; // junk value for setRed
enum redState {NaRed,KRed,CaRed};

int setCyan = 10;
enum cyanState {ScopeVoltageCyan,ScopeCurrentCyan};

int xValue = 0;
int prevXValue = 0;

double yVoltageValue = 0;
double prevYValue = 0;

double yNaValue;
double yKValue;
double yCaValue;

// Screen scaling variables
double VoltageRange = 240; 
double NaRange = 0.13;
double KRange = 1.4;
double CaRange = 5.36;

double VoltageMin = 0;
double NaMin = -0.17;
double KMin = 0.1;
double CaMin = -3;

double VoltageMax = 50;
double NaMax = -0.04;
double KMax = 0.2;
double CaMax = -1;

// character buffer arrays
char firstRowFirstColumnBuffer[10];
char secondRowFirstColumnBuffer[10];
char thirdRowFirstColumnBuffer[10];

char firstRowSecondColumnBuffer[10];
char secondRowSecondColumnBuffer[10];
char thirdRowSecondColumnBuffer[10];

char tempFirstRowFirstColumnBuffer[10];
char tempSecondRowFirstColumnBuffer[10];
char tempThirdRowFirstColumnBuffer[10];

char tempFirstRowSecondColumnBuffer[10];
char tempSecondRowSecondColumnBuffer[10];
char tempThirdRowSecondColumnBuffer[10];

// temporary conductance and concentration variables

double tempNAe = 0;
double tempCAe = 0; 
double tempKe = 0; 

double tempGNA = 0;
double tempGCA = 0;
double tempGK = 0;

// oscilloscope voltage and current arrays
double voltageArray[320];
double pixelVoltageArray[320];

double currentArray[320];
double pixelCurrentArray[320];

double kArray[320];
int kArrayIndex = 0;

double NaArray[320];
int NaArrayIndex = 0;

double CaArray[320];
int CaArrayIndex = 0;

char testVoltageArrayValue[10];
char testMembraneVoltage[10];

char test1[10];
char test2[10];
char test3[10];

// pause screen variable
// 0 is not paused, 1 is paused.
int pauseScreen = 0;

// scope output variable
// 0 is membrane voltage, 1 is total current
int scopeOutput = 0;

// cursor variables
int cursor = 1;
int cursorX;
int prevCursorX;
int cursor2X;
int prevCursor2X;
int cursorY = 0;
double timeValue;
double timeValue2;
double dTimeValue;

void calculateAdc(){
    // read the ADC from pin 26 (AN9)
    // read the first buffer position
    adc_10 = ReadADC10(0);   // read the result of channel 10 conversion from the idle buffer
    AcquireADC10(); // not needed if ADC_AUTO_SAMPLING_ON below

    // convert to voltage
    V = (float) adc_10 * 3.3 / 1023.0 ; // Vref*adc/1023
    // convert to fixed voltage
    Vfix = multfix16(float2fix16(adc_10), ADC_scale) ;
}

void checkUpDownButtons(int * pauseScreen, int * screen){
    if (downArrowValue == 1){
        centerButtonValue = 0;
        downArrowValue = 0;
        if (*pauseScreen == 0){
            *pauseScreen = 1;
            cursorX = 0;
            cursor2X = 20;
            prevCursorX = -1;
            prevCursor2X = 19;
        }
        else{
            tft_drawFastVLine(cursorX,cursorY,240,ILI9340_BLACK);
            tft_drawFastVLine(cursor2X,cursorY,240,ILI9340_BLACK);
            tft_drawPixel(cursorX,pixelVoltageArray[cursorX], ILI9340_RED);
            tft_drawPixel(cursorX,pixelCurrentArray[cursorX], ILI9340_GREEN);
            *pauseScreen = 0;
        }
    }
    
    if (*pauseScreen == 1 && upArrowValue == 1){
        upArrowValue = 0;
        if (cursor == 1){
            cursor = 2;
        }
        else if (cursor == 2){
            cursor = 1;
        }
    }
    
    if (*pauseScreen == 1 && centerButtonValue == 1){
        memset(voltageArray, 0, 320*sizeof(voltageArray[0]));
        memset(currentArray, 0, 320*sizeof(currentArray[0]));
        memset(pixelVoltageArray, 1000, 320*sizeof(pixelVoltageArray[0]));
        memset(pixelCurrentArray, 1000, 320*sizeof(pixelCurrentArray[0]));
        centerButtonValue = 0;
        upArrowValue = 0;
        downArrowValue = 0;
        leftArrowValue = 0;
        rightArrowValue = 0;
        clearScreen = 1;
        xValue = 0;
        *screen = 1;
    }
}

void clearScreenOnce(){
    if (clearScreen == 1){
        tft_fillScreen(ILI9340_BLACK);
        numberOfButtonPresses = 4;
        clearScreen = 0;
    }
}

void debounce(int* pushState, int* buttonState, int* buttonValue)
{
    switch(*pushState){
        case NoPush:
            if (*buttonState==0) *pushState = MaybePush;
            else *pushState = NoPush;
            break;
            
        case MaybePush:
            if (*buttonState==0) *pushState = Pushed;
            else *pushState = NoPush;
            break;
        
        case Pushed:
            if (*buttonState==0) *pushState = Pushed;
            else *pushState = MaybeNoPush;
            *buttonValue = 1;
            break;
           
        case MaybeNoPush:
            if (*buttonState==0) *pushState = Pushed;
            else *pushState = NoPush;
            break;
    }
}

void drawBlackBoxesFirstScreen(){
    tft_fillRoundRect(125, 25, 230, 15, 1, ILI9340_BLACK);// x,y,w,h,radius,color
    tft_fillRoundRect(125, 100, 230, 15, 1, ILI9340_BLACK);// x,y,w,h,radius,color
    tft_fillRoundRect(125, 175, 230, 15, 1, ILI9340_BLACK);// x,y,w,h,radius,color
}

void plotVoltage(){
    if (xValue > 319){
        xValue = 0;
        memset(voltageArray, 0, 320*sizeof(voltageArray[0]));
        memset(currentArray, 0, 320*sizeof(currentArray[0]));
        memset(pixelVoltageArray, 1000, 320*sizeof(pixelVoltageArray[0]));
        memset(pixelCurrentArray, 1000, 320*sizeof(pixelCurrentArray[0]));
        clearScreen = 1;
        clearScreenOnce();
    }
    
    if ((membraneVoltage > VoltageMax) || (membraneVoltage < VoltageMin)){
        if (membraneVoltage > VoltageMax){
            VoltageMax = membraneVoltage;
        }
        if (membraneVoltage < VoltageMin){ 
            VoltageMin = membraneVoltage;
        }
        VoltageRange = fabs(VoltageMax-VoltageMin);
    }
    yVoltageValue = (120/VoltageRange)*(-membraneVoltage+VoltageMax);
    //yVoltageValue = (VoltageMax)*(VoltageRange/120)+VoltageMax;
    voltageArray[xValue] = membraneVoltage;
    pixelVoltageArray[xValue] = yVoltageValue;
    tft_drawPixel(xValue, yVoltageValue, ILI9340_RED);
    
    // Draw Sodium, Potassium, or Calcium Current
    if (setRed == NaRed){
        if ((iNa > NaMax) || (iNa < NaMin)){
            if (iNa > NaMax){
                NaMax = iNa;
            }
            if ((iNa < NaMin) && (iNa > -1)){ // Occasional iNa values outside bound of reason. Bound to -1
                NaMin = iNa;
            }
            NaRange = fabs(NaMax-NaMin);
        }
        yNaValue = ((120/NaRange)*(-iNa+NaMax))+120;
        currentArray[xValue] = iNa;
        pixelCurrentArray[xValue] = yNaValue;
        tft_drawPixel(xValue, yNaValue, ILI9340_GREEN);      
    }
    if (setRed == KRed){
        yKValue = ((120/KRange)*(-iK+KMax))+120;
        if ((iK > KMax) || (iK < KMin)){
            if (iK > KMax){
                KMax = iK;
            }
            if ((iK < KMin)) {
                KMin = iK;
            }
            KRange = fabs(KMax-KMin);
        }
        currentArray[xValue] = iK;
        pixelCurrentArray[xValue] = yKValue;
        tft_drawPixel(xValue, yKValue, ILI9340_GREEN);
    }
    if (setRed == CaRed){
        yCaValue = ((120/CaRange)*(-iCa+CaMax))+120; // 22.3*(-iCa-0.14)+120;//(120/Range of y)*(-V+maxvalue)+120
        if ((iCa > CaMax) || (iCa < CaMin)){
            if (iCa > CaMax){
                CaMax = iCa;
            }
            if ((iCa < CaMin)) {
                CaMin = iCa;
            }
            CaRange = fabs(CaMax-CaMin);
        }
        currentArray[xValue] = iCa;
        pixelCurrentArray[xValue] = yCaValue;
        tft_drawPixel(xValue, yCaValue, ILI9340_GREEN);
    }
    
    xValue += 1;
}

void screenInit(){
  sprintf(firstRowFirstColumnBuffer,"%d",0);
  sprintf(secondRowFirstColumnBuffer,"%d",0);
  sprintf(thirdRowFirstColumnBuffer,"%d",0);
  sprintf(firstRowSecondColumnBuffer,"%d",0);
  sprintf(secondRowSecondColumnBuffer,"%d",0);
  sprintf(thirdRowSecondColumnBuffer,"%d",0);
  
  sprintf(tempFirstRowFirstColumnBuffer,"%d",0);
}

void updateFirstScreenConstantText(int screenRow){
    tft_setTextColor(ILI9340_YELLOW);
    tft_setCursor(0,25);
    tft_writeString("Sodium Channel");
    tft_setCursor(0,100);
    tft_writeString("Potassium Channel");
    tft_setCursor(0,175);
    tft_writeString("Calcium Channel");
    
    tft_setCursor(125,0);
    tft_writeString("Concentration");
    tft_setCursor(250,0);
    tft_writeString("Conductance");
    tft_setCursor(0,225);
    
    if (screenRow == 3){
        tft_setTextColor(ILI9340_BLUE);
        tft_writeString("Next Screen"); 
    }
    else{
        tft_writeString("Next Screen"); 
    }
}

void updateSecondScreenConstantText(int screenRow, int screenColumn){

    tft_setTextColor(ILI9340_YELLOW); 
    tft_setCursor(25, 15);
    tft_writeString("CHOOSE CURRENT");
    
    tft_setCursor (175, 15);
    tft_writeString("OSCILLOSCOPE OUTPUT");
    
    if (screenRow == 3 && screenColumn == 0){
        tft_setTextColor(ILI9340_BLUE);
        tft_setCursor(0,225);
        tft_writeString("Previous Screen"); 
    }
    else{
        tft_setTextColor(ILI9340_YELLOW);
        tft_setCursor(0,225);
        tft_writeString("Previous Screen");
    }
    
    if (screenRow == 3 && screenColumn == 1){
        tft_setTextColor(ILI9340_BLUE);
        tft_setCursor(225,225);
        tft_writeString("Run Simulation"); 
    }
    else{
        tft_setTextColor(ILI9340_YELLOW);
        tft_setCursor(225,225);
        tft_writeString("Run Simulation");         
    }
}

void updateScreenRow(int* screen){
    
    if (*screen == 1 && downArrowValue == 1 && screenRow == 2 && screenColumn == 1){
        screenRow +=1;
    }
    
    if (*screen == 1 && upArrowValue == 1 && screenRow == 3 && screenColumn == 1){
        screenRow -=1;
    }
    
    if (upArrowValue == 1 && screenRow > 0){ // Up Button has been pushed and not on first row
        screenRow -= 1;
        upArrowValue = 0;
    }
    
    if (downArrowValue == 1 && screenRow < 3){ // Down Button has been pushed and not on fourth row
        screenRow += 1;
        downArrowValue = 0;
    }
    
    if (rightArrowValue == 1 && screenColumn == 0){
        screenColumn = 1;
        rightArrowValue = 0;
    }
    
    if (leftArrowValue == 1 && screenColumn == 1){
        screenColumn = 0;
        leftArrowValue = 0;
    }

    if (*screen == 0){
        if (centerButtonValue == 1 && screenRow == 3){
            centerButtonValue = 0;
            calculateEquationConstants();
            clearScreen = 1;
            screenRow = 0;
            screenColumn = 0;
            rightArrowValue = 0;
            leftArrowValue = 0;
            downArrowValue = 0;
            upArrowValue = 0;
            *screen += 1;
        }
    }
    
    if (*screen == 1){
        if (centerButtonValue == 1){
            centerButtonValue = 0;
            if (screenRow < 3 && screenColumn == 0){
                setRed = screenRow;
            }
            if (screenRow < 2 && screenColumn == 1){
                setCyan = screenRow;
                scopeOutput = setCyan;
            }
            if (screenRow == 3 && screenColumn == 1){
                if (setRed<3 && setCyan <2){
                    leftArrowValue = 0;
                    rightArrowValue = 0;
                    upArrowValue = 0;
                    downArrowValue = 0;
                    clearScreen = 1;
                    *screen = 2;
                }
                else{
                    clearScreen = 1;
                    *screen = 3; // Error Screen
                }
            }
            
            if (screenRow == 3 && screenColumn == 0){
                clearScreen = 1;
                setRed = 10;
                setCyan = 10;
                *screen = 0;
            }
        }
    }
    
    if (*screen == 2){ // Transition Screen
        if (downArrowValue == 1 || leftArrowValue == 1 || rightArrowValue == 1){
            downArrowValue = 0;
            centerButtonValue = 0;
            leftArrowValue = 0;
            rightArrowValue = 0;       
            clearScreen = 1;
            clearScreenOnce();
            *screen = 4;
        }
    }
    
    if (*screen == 3){ // Error Screen
        if (upArrowValue == 1 || downArrowValue == 1 || leftArrowValue == 1 || rightArrowValue == 1){
            upArrowValue = 0;
            downArrowValue = 0;
            centerButtonValue = 0;
            leftArrowValue = 0;
            rightArrowValue = 0;       
            clearScreen = 1;
            *screen = 1;
        }
    }
}

void updateFirstScreen(){
    
    if (screenRow == 0 && screenColumn == 0){ // Sodium concentration
        tft_setTextColor(ILI9340_RED); 
        tft_setCursor(150, 25);
        
        // Calculate and print Sodium concentration
        if (centerButtonValue == 1){
            NAe = ((((double) adc_10) - 1023.0)*(190.0/1023.0)) + 200.0;
            sprintf(firstRowFirstColumnBuffer, "%0.3f", NAe); 
            tft_writeString(firstRowFirstColumnBuffer);
            centerButtonValue = 0;
        }
        else{
            tempNAe = ((((double) adc_10) - 1023.0)*(190.0/1023.0)) + 200.0;
            sprintf(tempFirstRowFirstColumnBuffer, "%0.3f", tempNAe); 
            tft_writeString(tempFirstRowFirstColumnBuffer);
        }
        
        // Write to constant values
        updateFirstScreenConstantText(screenRow);
                
        tft_setCursor(150, 100);
        tft_writeString(secondRowFirstColumnBuffer);
        tft_setCursor(150, 175);
        tft_writeString(thirdRowFirstColumnBuffer);
        
        tft_setCursor(275, 25);
        tft_writeString(firstRowSecondColumnBuffer);
        tft_setCursor(275, 100);
        tft_writeString(secondRowSecondColumnBuffer);
        tft_setCursor(275, 175);
        tft_writeString(thirdRowSecondColumnBuffer);
    }

    if (screenRow == 1 && screenColumn == 0){ // Potassium concentration
        tft_setTextColor(ILI9340_RED); 
        tft_setCursor(150, 100);
        
        // Calculate and print Potassium concentration
        if (centerButtonValue == 1){
            Ke = ((((double) adc_10) - 1023.0)*(1.4/1023.0)) + 4.3;
            sprintf(secondRowFirstColumnBuffer, "%0.3f", Ke); 
            tft_writeString(secondRowFirstColumnBuffer);
            centerButtonValue = 0;
        }
        else{
            tempKe = ((((double) adc_10) - 1023.0)*(1.4/1023.0)) + 4.3;
            sprintf(tempSecondRowFirstColumnBuffer, "%0.3f", tempKe); 
            tft_writeString(tempSecondRowFirstColumnBuffer);    
        }

        // Write to constant values
        updateFirstScreenConstantText(screenRow);
              
        tft_setCursor(150, 25);
        tft_writeString(firstRowFirstColumnBuffer);
        tft_setCursor(150, 175);
        tft_writeString(thirdRowFirstColumnBuffer);
        
        tft_setCursor(275, 25);
        tft_writeString(firstRowSecondColumnBuffer);
        tft_setCursor(275, 100);
        tft_writeString(secondRowSecondColumnBuffer);
        tft_setCursor(275, 175);
        tft_writeString(thirdRowSecondColumnBuffer);
    }

    if (screenRow == 2 && screenColumn == 0){ // Calcium conductance
        tft_setTextColor(ILI9340_RED); 
        tft_setCursor(150, 175);
        
        // Calculate and print Calcium conductance
        if (centerButtonValue == 1){
            CAe = ((((double) adc_10) - 1023.0)*(98.5/1023.0)) + 100;
            sprintf(thirdRowFirstColumnBuffer, "%0.3f", CAe); 
            tft_writeString(thirdRowFirstColumnBuffer);
            centerButtonValue = 0;
        }
        else{
            tempCAe = ((((double) adc_10) - 1023.0)*(98.5/1023.0)) + 100;
            sprintf(tempThirdRowFirstColumnBuffer, "%0.3f", tempCAe); 
            tft_writeString(tempThirdRowFirstColumnBuffer);
        }
        
        // Write to constant values
        updateFirstScreenConstantText(screenRow);
              
        tft_setCursor(150, 25);
        tft_writeString(firstRowFirstColumnBuffer);
        tft_setCursor(150, 100);
        tft_writeString(secondRowFirstColumnBuffer);
        
        tft_setCursor(275, 25);
        tft_writeString(firstRowSecondColumnBuffer);
        tft_setCursor(275, 100);
        tft_writeString(secondRowSecondColumnBuffer);
        tft_setCursor(275, 175);
        tft_writeString(thirdRowSecondColumnBuffer);
    }
    
    if (screenRow == 0 && screenColumn == 1){ // Sodium Conductance
        tft_setTextColor(ILI9340_RED); 
        tft_setCursor(275, 25);
        
        // Calculate and print Sodium Conductance
        if (centerButtonValue == 1){
            GNA = ((((double) adc_10) - 1023.0)*(16.0/1023.0)) + 16.0;
            sprintf(firstRowSecondColumnBuffer, "%0.3f", GNA);
            tft_writeString(firstRowSecondColumnBuffer);
            centerButtonValue = 0;
        }
        else{
            tempGNA = ((((double) adc_10) - 1023.0)*(16.0/1023.0)) + 16.0;
            sprintf(tempFirstRowSecondColumnBuffer, "%0.3f", tempGNA);
            tft_writeString(tempFirstRowSecondColumnBuffer);
        }
        
        // Write to constant values
        updateFirstScreenConstantText(screenRow);
              
        tft_setCursor(150, 25);
        tft_writeString(firstRowFirstColumnBuffer);
        tft_setCursor(150, 100);
        tft_writeString(secondRowFirstColumnBuffer);
        tft_setCursor(150, 175);
        tft_writeString(thirdRowFirstColumnBuffer);
        
        tft_setCursor(275, 100);
        tft_writeString(secondRowSecondColumnBuffer);
        tft_setCursor(275, 175);
        tft_writeString(thirdRowSecondColumnBuffer);
    }
    
    if (screenRow == 1 && screenColumn == 1){ // Potassium Conductance
        tft_setTextColor(ILI9340_RED); 
        tft_setCursor(275, 100);
        
        // Calculate and print Potassium conductance
        if (centerButtonValue == 1){
            GK = ((((double) adc_10) - 1023.0)*(1.0/1023.0)) + 1.0;
            sprintf(secondRowSecondColumnBuffer, "%0.3f", GK); 
            tft_writeString(secondRowSecondColumnBuffer); 
            centerButtonValue = 0;
        }
        else{
            tempGK = ((((double) adc_10) - 1023.0)*(1.0/1023.0)) + 1.0;
            sprintf(tempSecondRowSecondColumnBuffer, "%0.3f", tempGK); 
            tft_writeString(tempSecondRowSecondColumnBuffer);
        }

        // Write to constant values
        updateFirstScreenConstantText(screenRow);
              
        tft_setCursor(150, 25);
        tft_writeString(firstRowFirstColumnBuffer);
        tft_setCursor(150, 100);
        tft_writeString(secondRowFirstColumnBuffer);
        tft_setCursor(150, 175);
        tft_writeString(thirdRowFirstColumnBuffer);
        
        tft_setCursor(275, 25);
        tft_writeString(firstRowSecondColumnBuffer);
        tft_setCursor(275, 175);
        tft_writeString(thirdRowSecondColumnBuffer);
    }
    
    if (screenRow == 2 && screenColumn == 1){ // Calcium Conductance
        tft_setTextColor(ILI9340_RED); 
        tft_setCursor(275, 175);
        
        // Calculate and print Calcium conductance
        if (centerButtonValue == 1){
            GCA = ((((double) adc_10) - 1023.0)*(0.1/1023.0)) + 0.1;
            sprintf(thirdRowSecondColumnBuffer, "%0.3f", GCA); 
            tft_writeString(thirdRowSecondColumnBuffer);     
            centerButtonValue = 0;
        }
        else{
            tempGCA = ((((double) adc_10) - 1023.0)*(0.1/1023.0)) + 0.1;
            sprintf(tempThirdRowSecondColumnBuffer, "%0.3f", tempGCA); 
            tft_writeString(tempThirdRowSecondColumnBuffer); 
        }


        // Write to constant values
        updateFirstScreenConstantText(screenRow);
              
        tft_setCursor(150, 25);
        tft_writeString(firstRowFirstColumnBuffer);
        tft_setCursor(150, 100);
        tft_writeString(secondRowFirstColumnBuffer);
        tft_setCursor(150, 175);
        tft_writeString(thirdRowFirstColumnBuffer);
        
        tft_setCursor(275, 25);
        tft_writeString(firstRowSecondColumnBuffer);
        tft_setCursor(275, 100);
        tft_writeString(secondRowSecondColumnBuffer);
    }

    if (screenRow == 3){ // fourth row  
        // Write to constant values
        updateFirstScreenConstantText(screenRow);
        
        tft_setTextColor(ILI9340_YELLOW);
        tft_setCursor(150, 25);
        tft_writeString(firstRowFirstColumnBuffer);
        tft_setCursor(150, 100);
        tft_writeString(secondRowFirstColumnBuffer);
        tft_setCursor(150, 175);
        tft_writeString(thirdRowFirstColumnBuffer);
        
        tft_setCursor(275, 25);
        tft_writeString(firstRowSecondColumnBuffer);
        tft_setCursor(275, 100);
        tft_writeString(secondRowSecondColumnBuffer);
        tft_setCursor(275, 175);
        tft_writeString(thirdRowSecondColumnBuffer);
    }
}

void updateSecondScreen(){
    if (screenRow == 0 && screenColumn == 0){ // Display Sodium Current
        tft_setTextColor(ILI9340_BLUE); 
        tft_setCursor(0, 45);
        tft_writeString("Display Sodium Current");

        // Write to constant values
        tft_setTextColor(ILI9340_YELLOW); 
        tft_setCursor(0, 110);
        tft_writeString("Display Potassium Current");
        tft_setCursor(0, 175);
        tft_writeString("Display Calcium Current");
        tft_setCursor(175, 45);
        tft_writeString("Display Membrane Voltage");
        tft_setCursor(175, 110);
        tft_writeString("Display Chosen Current");
        
        updateSecondScreenConstantText(screenRow,screenColumn);
    }

    if (screenRow == 1 && screenColumn == 0){ // Display Potassium Current
        tft_setTextColor(ILI9340_BLUE);
        tft_setCursor(0, 110);
        tft_writeString("Display Potassium Current");

        // Write to constant values
        tft_setTextColor(ILI9340_YELLOW);
        tft_setCursor(0, 45);
        tft_writeString("Display Sodium Current");
        tft_setCursor(0, 175);
        tft_writeString("Display Calcium Current");
        tft_setCursor(175, 45);
        tft_writeString("Display Membrane Voltage");
        tft_setCursor(175, 110);
        tft_writeString("Display Chosen Current");
        
        updateSecondScreenConstantText(screenRow,screenColumn);    
    }

    if (screenRow == 2 && screenColumn == 0){ // Display Calcium Current
        tft_setTextColor(ILI9340_BLUE);
        tft_setCursor(0, 175);
        tft_writeString("Display Calcium Current");

        // Write to constant values
        tft_setTextColor(ILI9340_YELLOW);
        tft_setCursor(0, 45);
        tft_writeString("Display Sodium Current");
        tft_setCursor(0, 110);
        tft_writeString("Display Potassium Current");
        tft_setCursor(175, 45);
        tft_writeString("Display Membrane Voltage");
        tft_setCursor(175, 110);
        tft_writeString("Display Chosen Current");
        
        updateSecondScreenConstantText(screenRow,screenColumn);
    }

    if (screenRow == 3 && screenColumn == 0){ // fourth row
        // Write to constant values
        tft_setTextColor(ILI9340_YELLOW);
        tft_setCursor(0, 45);
        tft_writeString("Display Sodium Current");
        tft_setCursor(0, 110);
        tft_writeString("Display Potassium Current");
        tft_setCursor(0, 175);
        tft_writeString("Display Calcium Current");
        tft_setCursor(175, 45);
        tft_writeString("Display Membrane Voltage");
        tft_setCursor(175, 110);
        tft_writeString("Display Chosen Current");
        
        updateSecondScreenConstantText(screenRow,screenColumn);
    }
    
    if (screenRow == 0 && screenColumn == 1){
        tft_setTextColor(ILI9340_BLUE);
        tft_setCursor(175, 45);
        tft_writeString("Display Membrane Voltage");
        
        // Write to constant values
        tft_setTextColor(ILI9340_YELLOW);
        tft_setCursor(0, 45);
        tft_writeString("Display Sodium Current");
        tft_setCursor(0, 110);
        tft_writeString("Display Potassium Current");
        tft_setCursor(0, 175);
        tft_writeString("Display Calcium Current");
        tft_setCursor(175, 110);
        tft_writeString("Display Chosen Current");  
        
        updateSecondScreenConstantText(screenRow,screenColumn); 
    }
    
    if (screenRow == 1 && screenColumn == 1){
        tft_setTextColor(ILI9340_BLUE);
        tft_setCursor(175, 110);
        tft_writeString("Display Chosen Current");
        
        // Write to constant values
        tft_setTextColor(ILI9340_YELLOW);
        tft_setCursor(0, 45);
        tft_writeString("Display Sodium Current");
        tft_setCursor(0, 110);
        tft_writeString("Display Potassium Current");
        tft_setCursor(0, 175);
        tft_writeString("Display Calcium Current");
        tft_setCursor(175, 45);
        tft_writeString("Display Membrane Voltage");
        
        updateSecondScreenConstantText(screenRow,screenColumn); 
    }
    
    if (screenRow == 3 && screenColumn == 1){
        // Write to constant values
        tft_setTextColor(ILI9340_YELLOW);
        tft_setCursor(0, 45);
        tft_writeString("Display Sodium Current");
        tft_setCursor(0, 110);
        tft_writeString("Display Potassium Current");
        tft_setCursor(0, 175);
        tft_writeString("Display Calcium Current");
        tft_setCursor(175, 45);
        tft_writeString("Display Membrane Voltage");
        tft_setCursor(175, 110);
        tft_writeString("Display Chosen Current");
        
        updateSecondScreenConstantText(screenRow,screenColumn); 
    }
    
    if (setRed == NaRed){
        tft_setTextColor(ILI9340_RED);
        tft_setCursor(0, 45);
        tft_writeString("Display Sodium Current");
    }
    
    if (setRed == KRed){
        tft_setTextColor(ILI9340_RED);
        tft_setCursor(0, 110);
        tft_writeString("Display Potassium Current");
    }
    
    if (setRed == CaRed){
        tft_setTextColor(ILI9340_RED);
        tft_setCursor(0, 175);
        tft_writeString("Display Calcium Current");
    }
    
    if (setCyan == ScopeVoltageCyan){
        tft_setTextColor(ILI9340_CYAN);
        tft_setCursor(175, 45);
        tft_writeString("Display Membrane Voltage");
    }
    
    if (setCyan == ScopeCurrentCyan){
        tft_setTextColor(ILI9340_CYAN);
        tft_setCursor(175, 110);
        tft_writeString("Display Chosen Current");
    }
}

void updateThirdScreen(){
    tft_setTextColor(ILI9340_YELLOW);
    tft_setCursor(0,25);
    tft_writeString("Use up/down arrows to stop and resume simulation");
    tft_setCursor(0,50);
    tft_writeString("When stopped, press center button to reset parameters");
    tft_setCursor(0,75);
    tft_writeString("Use left/right arrows to move cursor on traces");
    tft_setCursor(0,100);
    tft_writeString("Press any directional arrow to continue");
}

void checkCursorButtons(){
    if (cursor == 1){
        if (leftArrowValue == 1){
            leftArrowValue = 0;
            if (cursorX > 0){
                prevCursorX = cursorX;
                cursorX -= 1;
            }
        }
        if (rightArrowValue == 1){
            rightArrowValue = 0;
            if (cursorX < 319){
                prevCursorX = cursorX;
                cursorX += 1;
            }
        }
    }
    if (cursor == 2){
        if (leftArrowValue == 1){
            leftArrowValue = 0;
            if (cursor2X > 0){
                prevCursor2X = cursor2X;
                cursor2X -= 1;
            }
        }
        if (rightArrowValue == 1){
            rightArrowValue = 0;
            if (cursor2X < 319){
                prevCursor2X = cursor2X;
                cursor2X += 1;
            }
        }
    }
}

void updateFourthScreen(){
    tft_drawFastVLine(prevCursorX,cursorY,240,ILI9340_BLACK);
    tft_drawFastVLine(prevCursor2X,cursorY,240,ILI9340_BLACK);
    
    tft_drawPixel(prevCursorX,pixelVoltageArray[prevCursorX], ILI9340_RED);
    tft_drawPixel(prevCursorX,pixelCurrentArray[prevCursorX], ILI9340_GREEN); 
    
    tft_drawPixel(prevCursor2X,pixelVoltageArray[prevCursor2X], ILI9340_RED);
    tft_drawPixel(prevCursor2X,pixelCurrentArray[prevCursor2X], ILI9340_GREEN);
    
    tft_drawFastVLine(cursorX,cursorY,240,ILI9340_BLUE); 
    tft_drawFastVLine(cursor2X,cursorY,240,ILI9340_BLUE);
    
    timeValue = ((float) cursorX)*deltaTime*25.0;
    timeValue2 = ((float) cursor2X)*deltaTime*25.0;
    dTimeValue = fabs(timeValue-timeValue2);
    
    if (cursor == 1){
        
        tft_fillRoundRect(250, 0, 300, 15, 1, ILI9340_BLACK);
        tft_setCursor(250,0);
        sprintf(test1, "T: %0.3f", timeValue);
        tft_writeString(test1);

        tft_fillRoundRect(250, 20, 300, 15, 1, ILI9340_BLACK);
        tft_setCursor(250,20);
        sprintf(test1, "V: %0.3f", voltageArray[cursorX]);
        tft_writeString(test1);

        tft_fillRoundRect(250, 40, 300, 15, 1, ILI9340_BLACK);
        tft_setCursor(250,40);
        sprintf(test1, "I: %0.3f", currentArray[cursorX]);
        tft_writeString(test1);    
        
        tft_fillRoundRect(250, 60, 300, 15, 1, ILI9340_BLACK);
        tft_setCursor(250,60);
        sprintf(test1, "dT: %0.3f", dTimeValue);
        tft_writeString(test1);
    }
    
    if (cursor == 2){
        
        tft_fillRoundRect(250, 0, 300, 15, 1, ILI9340_BLACK);
        tft_setCursor(250,0);
        sprintf(test1, "T: %0.3f", timeValue2);
        tft_writeString(test1);

        tft_fillRoundRect(250, 20, 300, 15, 1, ILI9340_BLACK);
        tft_setCursor(250,20);
        sprintf(test1, "V: %0.3f", voltageArray[cursor2X]);
        tft_writeString(test1);

        tft_fillRoundRect(250, 40, 300, 15, 1, ILI9340_BLACK);
        tft_setCursor(250,40);
        sprintf(test1, "I: %0.3f", currentArray[cursor2X]);
        tft_writeString(test1);
        
        tft_fillRoundRect(250, 60, 300, 15, 1, ILI9340_BLACK);
        tft_setCursor(250,60);
        sprintf(test1, "dT: %0.3f", dTimeValue);
        tft_writeString(test1);
    }

}